using System;
using System.Windows;
using System.Windows.Controls;

namespace WpfPresentation.DataModel
{
    /// <summary>
    /// Represents a page of content included in a Topic.
    /// </summary>
    public class TopicPage : TocItem
    {
        #region Data

        readonly Topic topic;

        #endregion // Data

        #region Constructor

        /// <summary>
        /// Initializes a new instance of TopicPage.
        /// </summary>
        /// <param name="topic">The Topic which contains this content.</param>
        /// <param name="pageUri">The Uri of the associated Page resource.</param>        
        public TopicPage(Topic topic, Uri pageUri)
            : base(topic.TableOfContents)
        {
            if (topic == null)
                throw new ArgumentNullException("topic");

            if (pageUri == null)
                throw new ArgumentNullException("pageUri");

            this.topic = topic;

            base.Initialize(pageUri);            
        }

        #endregion // Constructor

        #region Public Properties

        #region Title

        /// <summary>
        /// Returns the title for of Topic which contains this page.
        /// </summary>
        public override string Title
        {
            get { return this.topic.Title; }
        }

        #endregion // Title

        #endregion // Public Properties
    }
}