using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using WpfPresentation.DataModel;
using System.Windows.Threading;

namespace WpfPresentation
{
    public partial class MainWindow : System.Windows.Window
    {
        public MainWindow()
        {
            InitializeComponent();

            this.AddHandler(
                TreeViewItem.ExpandedEvent,
                new RoutedEventHandler(OnTreeViewItemExpanded));
        }

        #region OnTreeViewItemExpanded

        private delegate void NoArgDelegate();

        /// <summary>
        /// Ensures that only one topic node is expanded at a time.
        /// If the expanded node is not selected, it will be selected.
        /// </summary>
        void OnTreeViewItemExpanded(object sender, RoutedEventArgs e)
        {
            // Make sure that the root TreeViewItems are fully generated.
            ItemContainerGenerator generator = this.navTree.ItemContainerGenerator;
            if (generator.Status != GeneratorStatus.ContainersGenerated)
                return;

            // Get the TreeViewItem which was just expanded.
            TreeViewItem expandedTopicItem = e.OriginalSource as TreeViewItem;
            if (expandedTopicItem == null )
                return;

            // If a TreeViewItem in a demo app is being affected, abort.
            Topic expandedTopic = this.navTree.ItemContainerGenerator.ItemFromContainer(expandedTopicItem) as Topic;
            if (expandedTopic == null)
                return;

            // Collapse the previously expanded TreeViewItem.
            foreach (Topic topic in this.navTree.Items)
            {
                TreeViewItem topicItem = generator.ContainerFromItem(topic) as TreeViewItem;

                if (topicItem != null &&
                    topicItem != expandedTopicItem &&
                    topicItem.IsExpanded)
                {
                    topicItem.IsExpanded = false;
                    break;
                }
            }

            // Make sure the newly expanded topic node is selected.
            // This action needs to be postponed because of timing issues
            // regarding TreeViewItem selection.
            expandedTopicItem.Dispatcher.BeginInvoke(
                DispatcherPriority.Background,
                (NoArgDelegate)delegate
                {
                    if (!expandedTopicItem.IsSelected)
                        expandedTopicItem.IsSelected = true;
                });
        }

        #endregion // OnTreeViewItemExpanded
    }
}