using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;

namespace WpfPresentation.MainWindowResources
{
    public partial class MainWindowDataTemplates : ResourceDictionary
    {
        /// <summary>
        /// Binds the TreeViewItem's IsSelected property to the IsCurrentItem
        /// property of the TocItem which it represents.
        /// </summary>
        void OnTreeViewItemRootElementLoaded(object sender, RoutedEventArgs e)
        {
            TreeViewItem treeViewItem = FindAncestorTreeViewItem(sender as DependencyObject);
            if (treeViewItem != null)
            {
                treeViewItem.SetBinding(
                    TreeViewItem.IsSelectedProperty,
                    new Binding("IsCurrentItem"));
            }
        }

        /// <summary>
        /// Walks up the visual tree, starting at 'depObj', and returns the
        /// first TreeViewItem it finds, or null.
        /// </summary>
        static TreeViewItem FindAncestorTreeViewItem(DependencyObject depObj)
        {
            while ((depObj = VisualTreeHelper.GetParent(depObj)) != null)
                if (depObj is TreeViewItem)
                    break;

            return depObj as TreeViewItem;
        }
    }
}