﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Data;
using System.Windows.Documents;
using System.IO;
using System.Xml;
using System.Windows.Markup;
using System.ComponentModel;
using CJC.Wpf.Converters;
using System.Xml.Linq;
using System.Windows.Controls;

namespace FlowDocumentPresentation.Topics.DataBinding
{
    internal class TemplatedToolTipViewModel : INotifyPropertyChanged
    {
        public TemplatedToolTipViewModel(RichTextBox rtBox)
        {
            this.rtBox = rtBox;
            LoadTemplateExampleSimple();
            FlowerList = new FlowerCollection();
            Flowers = new ListCollectionView(FlowerList);
            RefreshTemplate();
        }
        private RichTextBox rtBox;
        private FlowerCollection FlowerList { get; set; }
        public ListCollectionView Flowers { get; set; }
        public FlowDocument ToolTipTemplate {
            get
            {
                return toolTipTemplate;
            }
            set
            {
                if (toolTipTemplate != value)
                {
                    toolTipTemplate = value;
                    NotifyPropertyChanged("ToolTipTemplate");
                }
            }
        }
        private FlowDocument toolTipTemplate = null;


        public void LoadTemplateExampleSimple()
        {
            SetToolTipTemplate(Properties.Resources.TemplateSimple);
        }

        public void LoadTemplateExampleWithTable()
        {
            SetToolTipTemplate(Properties.Resources.TemplateWithTable);
        }

        private void SetToolTipTemplate(string xmlString)
        {
            using (TextReader stringReader = new StringReader(xmlString))
            {
                using (XmlReader xmlReader = new XmlTextReader(stringReader))
                {
                    XElement xEle = XElement.Load(xmlReader);
                    PrettyXmlConverter xmlConv = new PrettyXmlConverter();
                    ToolTipTemplate = xmlConv.Render(xEle);
                }
            }
            rtBox.Document = ToolTipTemplate;
            //rtBox.Document.Blocks.Clear();
            //rtBox.Document.Blocks.AddRange(ToolTipTemplate.Blocks);
        }

        public void RefreshTemplate()
        {
            //string flowXML = toolTipTempl;
            //for (int i = 0; i < FlowerList.Count; i++)
            //{
            //    Flower flowerObj = FlowerList[i];
            //    FlowDocument flowDoc = null;
            //    using (TextReader stringReader = new StringReader(flowXML))
            //    {
            //        using (XmlReader xmlReader = new XmlTextReader(stringReader))
            //        {
            //            flowDoc = (FlowDocument)XamlReader.Load(xmlReader);
            //        }
            //    }
            //    flowerObj.FlowDocTemplate = flowDoc;
            //    FlowerList[i] = flowerObj;
            //}
            foreach (var flower in FlowerList)
            {
                //FlowDocument flowDoc = null;
                //using (TextReader stringReader = new StringReader(ToolTipTemplate))
                //{
                //    using (XmlReader xmlReader = new XmlTextReader(stringReader))
                //    {
                //        flowDoc = (FlowDocument)XamlReader.Load(xmlReader);
                //    }
                //}
                flower.FlowDocTemplate = ToolTipTemplate;
            }
           Flowers.Refresh();
        }
        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        protected void NotifyPropertyChanged(string name)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(name));
            }
        }

        #endregion

    }
}
