﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Data;
using System.Windows.Documents;
using System.IO;
using System.Xml;
using System.Windows.Markup;
using System.ComponentModel;
using System.Windows;

namespace FlowDocumentPresentation.Topics.DataBinding
{
    internal class TemplatedToolTipViewModel : INotifyPropertyChanged
    {
        public TemplatedToolTipViewModel()
        {
            LoadTemplateExampleSimple();
            FlowerList = new FlowerCollection();
            Flowers = new ListCollectionView(FlowerList);
            RefreshTemplate();
        }
        private FlowerCollection FlowerList { get; set; }
        public ListCollectionView Flowers { get; set; }
        public string ToolTipTemplate {
            get
            {
                return toolTipTemplate;
            }
            set
            {
                if (toolTipTemplate != value)
                {
                    toolTipTemplate = value;
                    NotifyPropertyChanged("ToolTipTemplate");
                }
            }
        }
        private string toolTipTemplate = string.Empty;


        public void LoadTemplateExampleSimple()
        {
            ToolTipTemplate = Properties.Resources.TemplateSimple;
        }

        public void LoadTemplateExampleWithTable()
        {
            ToolTipTemplate = Properties.Resources.TemplateWithTable;
        }

        public void RefreshTemplate()
        {

			try
			{
				//string flowXML = toolTipTempl;
				//for (int i = 0; i < FlowerList.Count; i++)
				//{
				//    Flower flowerObj = FlowerList[i];
				//    FlowDocument flowDoc = null;
				//    using (TextReader stringReader = new StringReader(flowXML))
				//    {
				//        using (XmlReader xmlReader = new XmlTextReader(stringReader))
				//        {
				//            flowDoc = (FlowDocument)XamlReader.Load(xmlReader);
				//        }
				//    }
				//    flowerObj.FlowDocTemplate = flowDoc;
				//    FlowerList[i] = flowerObj;
				//}
				foreach (var flower in FlowerList)
				{
					FlowDocument flowDoc = null;
					using (TextReader stringReader = new StringReader(ToolTipTemplate))
					{
						using (XmlReader xmlReader = new XmlTextReader(stringReader))
						{
							flowDoc = (FlowDocument)XamlReader.Load(xmlReader);
						}
					}
					flower.FlowDocTemplate = flowDoc;
				}
				Flowers.Refresh();
			}
			catch (Exception ex)
			{
				MessageBox.Show("Error in Documnent!\n" + ex.Message);
			}
        }
        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        protected void NotifyPropertyChanged(string name)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(name));
            }
        }

        #endregion

    }
}
