﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Globalization;
using System.Windows.Documents;
using System.Reflection;
using System.Windows.Controls;

namespace FlowDocumentPresentation.Topics.Writer
{
	internal class FindManager
	{


		private static Assembly frameworkAssembly;     // current assembly (for reflection of Find method)
		private static Type findEngineType;            // Type we're looking for
		private static BindingFlags flags;             // params for reflection
		private static MethodInfo findMethod;          // located Find method

		static FindManager()
		{
			frameworkAssembly = typeof(RichTextBox).Assembly;
			findEngineType = frameworkAssembly.GetType("System.Windows.Documents.TextFindEngine", true, false);
			flags = BindingFlags.InvokeMethod | BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Static;
			findMethod = findEngineType.GetMethod("Find", flags);
			if (findMethod == null)
			{
				throw new ApplicationException(@"Find method on TextFindEngine type not found");
			}
		}



		
		public static TextRange Find(TextPointer tpSstart,
			TextPointer tpEnd,
			string searchPattern,
			FindOptions findFlags,
			CultureInfo cultureInfo)
		{
			object[] args = new object[] { tpSstart, tpEnd, searchPattern, (int)findFlags, cultureInfo };
			TextRange rangeFound = findMethod.Invoke(null, args) as TextRange;
			return rangeFound;
		}





	}
}
