﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Win32;
using System.Windows.Markup;
using System.IO;
using Petzold.FlowDocumentHelp;
using System.Windows.Xps.Serialization;
using System.Windows.Xps.Packaging;
using System.IO.Packaging;
using FlowDocumentPresentation.Topics.Writer.Printing;
using System.Windows.Xps;
using System.Windows.Documents.Serialization;
using System.Printing;
using System.Globalization;

namespace FlowDocumentPresentation.Topics.Writer
{
	/// <summary>
	/// Interaction logic for SaveLoad.xaml
	/// </summary>
	public partial class RichTextBoxEditor : Page
	{
		public RichTextBoxEditor()
		{
			InitializeComponent();
			this.MainGrid.CommandBindings.Add(new CommandBinding(ApplicationCommands.Help, HelpOnExecuted));
			this.MainGrid.CommandBindings.Add(new CommandBinding(ApplicationCommands.Print, PrintOnExecuted));
			this.mnuItemOpen.Click += new RoutedEventHandler(mnuItemOpen_Click);
			this.mnuItemSaveAs.Click += new RoutedEventHandler(mnuItemSaveAs_Click);
			this.mnuItemSaveAsXPS.Click += new RoutedEventHandler(mnuItemSaveAsXPS_Click);

		}



		void mnuItemSaveAs_Click(object sender, RoutedEventArgs e)
		{
			SaveFileDialog saveFile = new SaveFileDialog();
			FileStream xamlFile = null;
			saveFile.AddExtension = true;
			saveFile.DefaultExt = "xaml";
			saveFile.OverwritePrompt = true;
			saveFile.Filter = "FlowDocument Files (*.xaml)|*.xaml|All Files (*.*)|*.*";
			saveFile.Title = "Save as FlowDocument";
			if (saveFile.ShowDialog(null).Value)
			{
				try
				{
					xamlFile = saveFile.OpenFile() as FileStream;
				}
				catch (Exception ex)
				{
					String error = "There was a opening the file:\n\n";
					error += saveFile.FileName;
					error += "\n\nException details:\n\n";
					error += ex.Message;
					System.Windows.MessageBox.Show(error);
					return;
				}
				if (xamlFile == null) return;
				else
				{
					XamlWriter.Save(mainRTB.Document, xamlFile);
					xamlFile.Close();
				}
			}

		}

		void mnuItemOpen_Click(object sender, RoutedEventArgs e)
		{
			FlowDocument content = null;
			OpenFileDialog openFileDialog = new OpenFileDialog();
			openFileDialog.AddExtension = true;
			openFileDialog.CheckFileExists = true;
			openFileDialog.CheckPathExists = true;
			openFileDialog.DefaultExt = "xaml";
			openFileDialog.Filter = "Flow Document (*.xaml)|*.xaml|All files (*.*)|*.*";
			openFileDialog.Multiselect = false;
			openFileDialog.Title = "Load Flow Document";
			if (openFileDialog.ShowDialog().Value)
			{
				FileStream xamlFile = openFileDialog.OpenFile() as FileStream;
				if (xamlFile == null) return;
				else
				{
					try
					{
						ParserContext context = new ParserContext();
						context.BaseUri = new Uri(openFileDialog.FileName, UriKind.Absolute);
						content = XamlReader.Load(xamlFile, context) as FlowDocument;
						if (content == null)
							throw (new XamlParseException("The specified file could not be loaded as a FlowDocument."));
					}
					catch (XamlParseException ex)
					{
						String error = "There was a problem parsing the specified file:\n\n";
						error += openFileDialog.FileName;
						error += "\n\nException details:\n\n";
						error += ex.Message;
						System.Windows.MessageBox.Show(error);
						return;
					}
					catch (Exception ex2)
					{
						String error = "There was a problem loading the specified file:\n\n";
						error += openFileDialog.FileName;
						error += "\n\nException details:\n\n";
						error += ex2.Message;
						System.Windows.MessageBox.Show(error);
						return;
					}
					// At this point, there is a non-null FlowDocument loaded into content.  
					this.mainRTB.Document = content;
				}
			}
		}

		void HelpOnExecuted(object sender, ExecutedRoutedEventArgs args)
		{
			HelpWindow win = new HelpWindow();
			win.Owner = GetParentWindow();
			win.Title = "Help for Generic";
			win.Show();
		}

		Window parentWindow = null;
		private Window GetParentWindow()
		{
			if (parentWindow == null)
			{
				bool windowFound = false;
				DependencyObject dpObj = this.MainGrid;
				while (!windowFound)
				{
					dpObj = VisualTreeHelper.GetParent(dpObj);
					if ((dpObj is Window) || (dpObj == null))
					{
						windowFound = true;
						parentWindow = dpObj as Window;
					}
				}
			}
			return parentWindow;
		}

		void PrintOnExecuted(object sender, ExecutedRoutedEventArgs args)
		{
			bool isUseCustomePaginator = Convert.ToBoolean( args.Parameter);
			PrintDialog printDlg = new PrintDialog();
			bool? dialogResult = printDlg.ShowDialog();
			if (dialogResult == true)
			{
				PrintTicket printTicket = printDlg.PrintTicket;
				// In case the user has changed some settings (e.g., orientation or page size)
				// by going to the printer preferences dialog from the print dialog itself,
				// set the printer queue's User ticket to the one attached to the print dialog.
				printDlg.PrintQueue.UserPrintTicket = printTicket;

				// Print the FlowDocument
				FlowDocument flDoc = GetFlowDocumentForPrinting();
				IDocumentPaginatorSource paginatSource = (IDocumentPaginatorSource)flDoc;

				XpsDocumentWriter docWriter = PrintQueue.CreateXpsDocumentWriter(printDlg.PrintQueue);
				DocumentPaginator paginator = paginatSource.DocumentPaginator;
				if (isUseCustomePaginator)
					paginator = new DocumentPaginatorWrapper(paginator, new Size(768, 676), new Size(48, 48));
				docWriter.WritingCompleted += new WritingCompletedEventHandler(docWriter_WritingCompleted);
				docWriter.WriteAsync(paginator, printTicket);
			}
		}

		void docWriter_WritingCompleted(object sender, WritingCompletedEventArgs e)
		{
			Console.WriteLine("Printing completed");
		}
		


		void mnuItemSaveAsXPS_Click(object sender, RoutedEventArgs e)
		{
			SaveFileDialog saveFile = new SaveFileDialog();
			saveFile.AddExtension = true;
			saveFile.DefaultExt = "xps";
			saveFile.OverwritePrompt = true;
			saveFile.Filter = "FixDocument Files (*.xps)|*.xps|All Files (*.*)|*.*";
			saveFile.Title = "Save as FixDocument xps";
			if (saveFile.ShowDialog(null).Value)
			{
				try
				{
					FlowDocument flDoc = GetFlowDocumentForPrinting();
					IDocumentPaginatorSource paginatSource = (IDocumentPaginatorSource)flDoc;
					//DoSaveAsXps(saveFile.FileName, paginatSource);
					//WriteXPSDoc(saveFile.FileName, paginatSource);
					WriteXPSDocAsync(saveFile.FileName, paginatSource);
				}
				catch (Exception ex)
				{
					String error = "There was a opening the file:\n\n";
					error += saveFile.FileName;
					error += "\n\nException details:\n\n";
					error += ex.Message;
					System.Windows.MessageBox.Show(error);
					return;
				}
			}
		}
		Package xpsContainer = null;
		XpsDocument xpsDocument = null;
		private void WriteXPSDocAsync(string fileName, IDocumentPaginatorSource doc)
		{
			xpsContainer = Package.Open(fileName, FileMode.Create);
			xpsDocument = new XpsDocument(xpsContainer, CompressionOption.Maximum);
			XpsDocumentWriter xpsdw = XpsDocument.CreateXpsDocumentWriter(xpsDocument);
			DocumentPaginator paginator = doc.DocumentPaginator;
			paginator = new DocumentPaginatorWrapper(paginator, new Size(768, 676), new Size(48, 48));
			xpsdw.WritingCompleted += new WritingCompletedEventHandler(xpsdw_WritingCompleted);
			xpsdw.WriteAsync(paginator);
		}

		void xpsdw_WritingCompleted(object sender, WritingCompletedEventArgs e)
		{
			xpsDocument.Close();
			xpsContainer.Close();
			xpsDocument = null;
			xpsContainer = null;
			Console.WriteLine("XPS document created");
		}


		//private void DoSaveAsXps(string fileName, IDocumentPaginatorSource doc)
		//{
		//    using (Package container = Package.Open(fileName, FileMode.Create))
		//    {
		//        using (XpsDocument xpsDoc = new XpsDocument(container, CompressionOption.Maximum))
		//        {
		//            XpsSerializationManager rsm = new XpsSerializationManager(new XpsPackagingPolicy(xpsDoc), false);
		//            DocumentPaginator paginator = doc.DocumentPaginator;
		//            // 8 inch x 6 inch, with half inch margin
		//            //paginator = new DocumentPaginatorWrapper(paginator, new Size(768, 676), new Size(48, 48));
		//            rsm.SaveAsXaml(paginator);
		//        }
		//    }
		//    return ;
		//}
		//private void WriteXPSDoc(string fileName, IDocumentPaginatorSource doc)
		//{
		//    using (Package container = Package.Open(fileName, FileMode.Create))
		//    {
		//        using (XpsDocument xpsDoc = new XpsDocument(container, CompressionOption.Maximum))
		//        {
		//            XpsDocumentWriter xpsdw = XpsDocument.CreateXpsDocumentWriter(xpsDoc);
		//            DocumentPaginator paginator = doc.DocumentPaginator;
		//            paginator = new DocumentPaginatorWrapper(paginator, new Size(768, 676), new Size(48, 48));
		//            //xpsdw.WritingCompleted += new WritingCompletedEventHandler(AsyncSaveCompleted);
		//            xpsdw.Write(paginator);
		//        }
		//    }
		//}

		private FlowDocument GetFlowDocumentForPrinting()
		{
			FlowDocument flowDoc = new FlowDocument();

			TextRange flowDocTextRange = new TextRange(flowDoc.ContentStart, flowDoc.ContentEnd);
			TextRange rtbTextRange;
			rtbTextRange = new TextRange(this.mainRTB.Document.ContentStart, this.mainRTB.Document.ContentEnd);
			string _rangeXML = TextRange_GetXml(rtbTextRange);
			TextRange_SetXml(flowDocTextRange, _rangeXML);
			return flowDoc;

		}


		/// <summary>
		/// Get xaml from TextRange.Xml property
		/// </summary>
		/// <param name="range">TextRange</param>
		/// <returns>return a string serialized from the TextRange</returns>
		private string TextRange_GetXml(TextRange range)
		{
			MemoryStream mstream;

			if (range == null)
			{
				throw new ArgumentNullException("range");
			}

			mstream = new MemoryStream();
			range.Save(mstream, DataFormats.Xaml);

			//must move the stream pointer to the beginning since range.save() will move it to the end.
			mstream.Seek(0, SeekOrigin.Begin);

			//Create a stream reader to read the xaml.
			StreamReader stringReader = new StreamReader(mstream);

			return stringReader.ReadToEnd();
		}

		/// <summary>
		/// Set xml to TextRange.Xml property.
		/// </summary>
		/// <param name="range">TextRange</param>
		/// <param name="xml">Xaml to be set</param>        
		private void TextRange_SetXml(TextRange range, string xml)
		{
			MemoryStream mstream;
			if (null == xml)
			{
				throw new ArgumentNullException("xml");
			}
			if (range == null)
			{
				throw new ArgumentNullException("range");
			}

			mstream = new MemoryStream();
			StreamWriter sWriter = new StreamWriter(mstream);

			mstream.Seek(0, SeekOrigin.Begin); //this line may not be needed.
			sWriter.Write(xml);
			sWriter.Flush();

			//move the stream pointer to the beginning.
			mstream.Seek(0, SeekOrigin.Begin);

			range.Load(mstream, DataFormats.Xaml);
		}

		private void DoFind(string searchPattern)
		{
			TextRange rangeFound = FindManager.Find(this.mainRTB.Document.ContentStart, this.mainRTB.Document.ContentEnd,
				searchPattern, FindOptions.None, CultureInfo.CurrentCulture);
			if (rangeFound != null)
				this.mainRTB.Selection.Select(rangeFound.Start, rangeFound.End);
			this.mainRTB.Focus();
		}



	}
}
