using System;
using System.ComponentModel;
using System.IO;
using System.Windows;
using System.Windows.Data;
using System.Windows.Resources;

namespace WpfPresentation.Utilities
{
    /// <summary>
    /// A XAML-friendly class which makes all of the
    /// text in a resource file available.
    /// </summary>
    public class ResourceTextProvider : DataSourceProvider
    {
        string resourceFile;

        /// <summary>
        /// Gets/sets the path of a resource file 
        /// from which to get text.
        /// </summary>
        public string ResourceFile
        {
            get { return this.resourceFile; }
            set
            {
                if (value == this.resourceFile)
                    return;

                this.resourceFile = value;

                base.OnPropertyChanged(new PropertyChangedEventArgs("ResourceFile"));
            }
        }

        protected override void BeginQuery()
        {
            string text = null;

            // Load text out of the resource file.
            if (!String.IsNullOrEmpty(this.ResourceFile))
                text = this.GetResourceFileText();

            // Let the base class know that the data has been loaded.
            base.OnQueryFinished(text);
        }

        /// <summary>
        /// Returns all of the text in the resource file.
        /// </summary>
        string GetResourceFileText()
        {
            Uri fileUri = new Uri(this.ResourceFile, UriKind.Relative);
            StreamResourceInfo resInfo = Application.GetResourceStream(fileUri);
            using (StreamReader rdr = new StreamReader(resInfo.Stream))
                return rdr.ReadToEnd();
        }
    }
}